﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Policy;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media.Media3D;
using System.Xml.Linq;
using Tonwicht_Management.Classes;
using RestSharp;
using static System.Net.Mime.MediaTypeNames;

namespace Tonwicht_Management.Connectors
{
    internal class API_Connector
    {
        private readonly string _url = "https://api.tonwicht.de";
        private RestClient _client = null;

        public API_Connector()
        {
            this._client = new RestClient(new RestClientOptions(this._url));
        }

        private async Task<RestResponse> Make_Response(string requestPath)
        {
            return await this._client.GetAsync(new RestRequest(requestPath));
        }

        public async Task<List<Waregroup>> GetWaregroups()
        {
            List<Waregroup> waregroups = new();

            dynamic obj = JsonConvert.DeserializeObject((await this.Make_Response("waregroups")).Content);

            foreach (dynamic waregroup in obj.data)
            {
                Waregroup w = new();

                w.Set_ID((int)waregroup.idwaregroups);
                w.Set_Name((string)waregroup.name);
                w.Set_Created((DateTime)waregroup.created);
                w.Set_Changed((DateTime)waregroup.changed);

                waregroups.Add(w);
            }
            return waregroups;
        }

        public async Task<List<Article>> Get_Articles()
        {
            List<Article> articles = new();
            dynamic obj = JsonConvert.DeserializeObject((await this.Make_Response("articles")).Content);
            foreach (dynamic waregroup in obj.data)
            {
                Article article = new();

                article.Set_ID((int)waregroup.idarticles);
                article.Set_Name((string)waregroup.name);
                article.Set_WaregroupID((int)waregroup.waregroup);
                article.Set_Size((double)waregroup.size);
                article.Set_Weight((double)waregroup.weight);
                article.Set_Price((double)waregroup.price);
                article.Set_Stock((int)waregroup.stock);
                article.Set_Created((DateTime)waregroup.created);
                article.Set_Changed((DateTime)waregroup.changed);

                articles.Add(article);
            }
            return articles;
        }

        public async Task<int> Insert_NewArticle(string name, int waregroupID, double size, double price, double weight)
        {
            Dictionary<string, object> values = new()
            {
                { "name" , name },
                { "waregroupID" , waregroupID},
                { "size" , size},
                { "price", price},
                { "weight" , weight }
            };

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(this._url + "/articles");
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "POST";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                string json = JsonConvert.SerializeObject(values);

                streamWriter.Write(json);
            }

            var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            using var streamReader = new StreamReader(httpResponse.GetResponseStream());
            var result = streamReader.ReadToEnd();

            dynamic x = JsonConvert.DeserializeObject(result) ?? String.Empty;

            return x.newArticleID;
        }

        public void Delete_Article(int articleID)
        {
            Dictionary<string, object> values = new()
            {
                { "articleID" , articleID }
            };

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(this._url + "/articles/" + articleID);
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "DELETE";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                string json = JsonConvert.SerializeObject(values);

                streamWriter.Write(json);
            }

            var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            using var streamReader = new StreamReader(httpResponse.GetResponseStream());
            var result = streamReader.ReadToEnd();

            dynamic x = JsonConvert.DeserializeObject(result) ?? String.Empty;
        }

        public void Insert_Waregroup(string waregroupName)
        {
            Dictionary<string, object> values = new()
            {
                { "name" , waregroupName }
            };

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(this._url + "/waregroups");
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "POST";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                string json = JsonConvert.SerializeObject(values);

                streamWriter.Write(json);
            }

            var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            using var streamReader = new StreamReader(httpResponse.GetResponseStream());
            var result = streamReader.ReadToEnd();

            dynamic x = JsonConvert.DeserializeObject(result) ?? String.Empty;
        }

        public void Edit_Waregroup(int waregroupID, string newName)
        {
            Dictionary<string, object> values = new()
            {
                { "name" , newName }
            };

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(this._url + "/waregroups/" + waregroupID);
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "PATCH";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                string json = JsonConvert.SerializeObject(values);

                streamWriter.Write(json);
            }

            var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            using var streamReader = new StreamReader(httpResponse.GetResponseStream());
            var result = streamReader.ReadToEnd();

            dynamic x = JsonConvert.DeserializeObject(result) ?? String.Empty;
        }

        public void Delete_Waregroup(int waregroupID)
        {
            Dictionary<string, object> values = new()
            {
                { "waregroupID" , waregroupID }
            };

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(this._url + "/waregroups/" + waregroupID );
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "DELETE";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                string json = JsonConvert.SerializeObject(values);

                streamWriter.Write(json);
            }

            var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            using var streamReader = new StreamReader(httpResponse.GetResponseStream());
            var result = streamReader.ReadToEnd();

            dynamic x = JsonConvert.DeserializeObject(result) ?? String.Empty;
        }
    }
}
