﻿using MahApps.Metro.Controls;
using MahApps.Metro.Controls.Dialogs;
using Microsoft.Win32;
using Renci.SshNet.Sftp;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Tonwicht_Management.Classes;
using Tonwicht_Management.Connectors;

namespace Tonwicht_Management.Frontend
{
    /// <summary>
    /// Interaktionslogik für AddArticle.xaml
    /// </summary>
    public partial class AddArticle : MetroWindow
    {
        private readonly DialogOpeningMode _openMode;
        private readonly Article _article;

        public AddArticle(DialogOpeningMode openingMode, Article articleData)
        {
            InitializeComponent();

            this._openMode = openingMode;
            this._article = articleData;

            this.LoadWaregroups();

            switch (openingMode)
            {
                case DialogOpeningMode.add:
                    this.Title = "Neuen Artikel hinzufügen";
                    break;
                case DialogOpeningMode.edit:
                    this.Title = "Artikel bearbeiten";
                    this.LoadArticleDataToForm();
                    break;
                case DialogOpeningMode.delete:
                    this.Title = "Artikel löschen";
                    this.LoadArticleDataToForm();
                    this.btn_save.Content = "Löschen";

                    this.tbx_name.IsEnabled = false;
                    this.tbx_price.IsEnabled = false;
                    this.tbx_size.IsEnabled = false;
                    this.tbx_weight.IsEnabled = false;
                    this.btn_select_image_front.IsEnabled = false;
                    this.btn_select_image_back.IsEnabled = false;
                    this.cbx_waregroup.IsEnabled = false;
                    break;
            }
        }

        private void LoadWaregroups()
        {
            foreach (Waregroup wg in Cache.Waregroups)
            {
                this.cbx_waregroup.Items.Add(wg.GetName());
            }
        }

        private void LoadArticleDataToForm()
        {
            this.tbx_name.Text = this._article.Name;
            this.tbx_size.Text = this._article.Size.ToString();
            this.tbx_weight.Text = this._article.Weight.ToString();
            this.tbx_price.Text = this._article.Price.ToString();

            foreach (Waregroup wg in Cache.Waregroups)
            {
                if(wg.Get_ID() == this._article.Get_WaregroupID())
                {
                    foreach (var item in this.cbx_waregroup.Items)
                    {
                        if (wg.GetName() == item.ToString())
                        {
                            this.cbx_waregroup.SelectedItem = item;
                            break;
                        }
                    }
                }                
            }

            string filePath_FrontView = this._article.LocalImagePathFront;
            string filePath_BackView = this._article.LocalImagePathBack;

            if(filePath_FrontView != "")
            {
                BitmapImage bitmap = new();
                bitmap.BeginInit();
                bitmap.UriSource = new Uri(filePath_FrontView);
                bitmap.EndInit();
                img_front.Source = bitmap;
            }

            if(filePath_BackView != "")
            {
                BitmapImage bitmap_back = new();
                bitmap_back.BeginInit();
                bitmap_back.UriSource = new Uri(filePath_BackView);
                bitmap_back.EndInit();
                img_back.Source = bitmap_back;
            }            
        }

        private void Btn_select_image_front_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new()
            {
                Multiselect = false
            };
            if (openFileDialog.ShowDialog() == true)
            {
                BitmapImage bitmap = new();
                bitmap.BeginInit();
                bitmap.UriSource = new Uri(openFileDialog.FileName);
                bitmap.EndInit();
                img_front.Source = bitmap;
                img_front.Tag = openFileDialog.FileName;
            }
        }

        private void Btn_select_image_back_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new()
            {
                Multiselect = false
            };
            if (openFileDialog.ShowDialog() == true)
            {
                BitmapImage bitmap = new();
                bitmap.BeginInit();
                bitmap.UriSource = new Uri(openFileDialog.FileName);
                bitmap.EndInit();
                img_back.Source = bitmap;
                img_back.Tag = openFileDialog.FileName;
            }
        }

        private async void Btn_save_Click(object sender, RoutedEventArgs e)
        {
            API_Connector api = new();
            SFTP_Connector connector = new();

            if (this._openMode == DialogOpeningMode.add)
            {
                if (
                    this.tbx_name.Text != "" &&
                     this.tbx_price.Text != "" &&
                     this.tbx_size.Text != "" &&
                     this.tbx_weight.Text != "" &&
                     this.cbx_waregroup.SelectedIndex != -1
                    )
                {
                    int wgID = 0;

                    foreach (Waregroup wg in Cache.Waregroups)
                    {
                        if (wg.GetName() == this.cbx_waregroup.SelectedValue.ToString())
                        {
                            wgID = wg.Get_ID();
                        }
                    }

                    int insertedID = await api.Insert_NewArticle(this.tbx_name.Text, wgID, Convert.ToDouble(this.tbx_size.Text), Convert.ToDouble(this.tbx_price.Text), Convert.ToDouble(this.tbx_weight.Text));

                    string[] parts_front = img_front.Tag.ToString().Split(".");
                    string[] parts_back = img_back.Tag.ToString().Split(".");

                    string CacheDir = "Cache/";

                    if(!Directory.Exists(CacheDir)) Directory.CreateDirectory(CacheDir);
                    
                    NetVips.Image.Thumbnail(img_front.Tag.ToString(), 500).WriteToFile(CacheDir + "out_front.jpg");
                    NetVips.Image.Thumbnail(img_back.Tag.ToString(), 500).WriteToFile(CacheDir + "out_back.jpg");

                    connector.UploadFile(CacheDir + "out_front.jpg", "images/articles/" + insertedID + "_1." + parts_front[^1]);
                    connector.UploadFile(CacheDir + "out_back.jpg", "images/articles/" + insertedID + "_2." + parts_back[^1]);

                    MetroDialogSettings sets = new MetroDialogSettings
                    {
                        AffirmativeButtonText = "Ja",
                        NegativeButtonText = "Nein"
                    };

                    MessageDialogResult result = await this.ShowMessageAsync("Artikel erstellt", "Willst du noch einen Artikel erstellen?", MessageDialogStyle.AffirmativeAndNegative, sets);

                    if (result == MessageDialogResult.Affirmative)
                    {
                        this.tbx_name.Text = "";
                        this.tbx_price.Text = "";
                        this.tbx_size.Text = "";
                        this.tbx_weight.Text = "";
                        this.img_front.Source = null;
                        this.img_front.Tag = null;
                        this.img_back.Source = null;
                        this.img_back.Tag = null;
                    }
                    else
                    {
                        this.Close();
                    }
                }
                else
                {
                    await this.ShowMessageAsync("Es ist ein Fehler aufgetreten", "Es fehlen Daten");
                }
            }

            else if(this._openMode == DialogOpeningMode.edit)
            {

            }

            else if(this._openMode == DialogOpeningMode.delete)
            {
                List<SftpFile> files = connector.Get_ArticleImages();
                foreach (SftpFile file in files)
                {
                    if (file.Name.StartsWith(this._article.Get_ID() + "_")) connector.DeleteFile("images/articles/" + file.Name);
                }

                api.Delete_Article(this._article.Get_ID());                
                this.Close();
            }
        }

        private void btn_abort_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
