﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using AutoUpdaterDotNET;
using MahApps.Metro.Controls;
using MahApps.Metro.Controls.Dialogs;
using Newtonsoft.Json;
using Renci.SshNet;
using Renci.SshNet.Sftp;
using Tonwicht_Management.Classes;
using Tonwicht_Management.Connectors;
using Tonwicht_Management.Frontend;

namespace Tonwicht_Management
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : MetroWindow
    {
        private readonly API_Connector api = new();

        public MainWindow()
        {
            InitializeComponent();
            this.FetchAllData();
        }

        #region Form
        private void MetroWindow_Loaded(object sender, RoutedEventArgs e)
        {
            string localVersionString;

            if (!File.Exists("version.log"))
                localVersionString = "0.0.0";
            else
                localVersionString = File.ReadAllLines("version.log")[0];

            AutoUpdater.InstalledVersion = new Version(localVersionString);

            AutoUpdater.ParseUpdateInfoEvent += AutoUpdater_ParseUpdateInfoEvent;
            AutoUpdater.Start("https://cdn.patrickweigelt.com/Software/Tonwicht/Tonwicht-Management.json");

            this.lbl_version.Content = "Version: " + localVersionString;
        }

        private void AutoUpdater_ParseUpdateInfoEvent(ParseUpdateInfoEventArgs args)
        {
            dynamic json = JsonConvert.DeserializeObject(args.RemoteData) ?? String.Empty;
            args.UpdateInfo = new UpdateInfoEventArgs
            {
                CurrentVersion = json.version,
                ChangelogURL = json.changelog,
                DownloadURL = json.url
            };
        }
        #endregion

        private void FetchAllData()
        {
            this.Get_Waregroups();
            this.Get_Articles();
            Cache.Update_ArticleImages();
        }

        private async void Get_Waregroups()
        {
            Cache.Waregroups = await api.GetWaregroups();
        }

        private async void Get_Articles()
        {
            Cache.Articles = await api.Get_Articles();
        }

        private void TabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            TabItem selected = (TabItem)((TabControl)sender).SelectedItem;

            switch (selected.Header)
            {
                case "Startseite":
                    break;
                case "Artikel":
                    dgv_article_data.ItemsSource = Cache.Articles;
                    break;
                case "Gruppen":
                    dgv_groups_data.ItemsSource = Cache.Waregroups;
                    break;
            }
        }
        #region Articles
        private void Btn_article_new_Click(object sender, RoutedEventArgs e)
        {
            AddArticle dialog = new(DialogOpeningMode.add, null);
            dialog.ShowDialog();

            Get_Articles();
            SFTP_Connector connector = new();
            Cache.ArticleImages.Clear();
            foreach (SftpFile image in connector.Get_ArticleImages())
            {
                Cache.ArticleImages.Add(image.Name);
            }
            this.dgv_article_data.ItemsSource = Cache.Articles;
        }

        private void Btn_article_edit_Click(object sender, RoutedEventArgs e)
        {
            AddArticle dialog = new(DialogOpeningMode.edit, (Article)dgv_article_data.SelectedItem);
            dialog.ShowDialog();
        }

        private void Btn_article_delete_Click(object sender, RoutedEventArgs e)
        {
            AddArticle dialog = new(DialogOpeningMode.delete, (Article)dgv_article_data.SelectedItem);
            dialog.ShowDialog();
            Get_Articles();
            this.dgv_article_data.ItemsSource = Cache.Articles;
        }
        #endregion

        #region Waregroups
        private async void Btn_waregroup_new_Click(object sender, RoutedEventArgs e)
        {
            string input = await this.ShowInputAsync("Neue Warengruppe anlegen", "Name: ");
            if(input != null)
            {
                api.Insert_Waregroup(input);
                Get_Waregroups();
                dgv_groups_data.ItemsSource = Cache.Waregroups;
            }
        }

        private void Btn_waregroup_edit_Click(object sender, RoutedEventArgs e)
        {
            this.CreateEditWaregroupDialog();
        }

        private void dgv_groups_data_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            this.CreateEditWaregroupDialog();
        }

        private async void CreateEditWaregroupDialog()
        {
            if (dgv_groups_data.SelectedIndex != -1)
            {
                Waregroup wg = (Waregroup)dgv_groups_data.SelectedItem;

                MetroDialogSettings sets = new()
                {
                    DefaultText = wg.GetName()
                };

                string input = await this.ShowInputAsync("Warengruppe umbenennen", "Name: ", sets);
                if (input != wg.GetName() && input != null)
                {
                    api.Edit_Waregroup(wg.Get_ID(), input);
                    Get_Waregroups();
                    dgv_groups_data.ItemsSource = Cache.Waregroups;
                }
            }
            else
            {
                await this.ShowMessageAsync("Ein Fehler ist aufgetreten", "Es wurde keine Warengruppe ausgewählt");
            }
        }

        private async void Btn_waregroup_delete_Click(object sender, RoutedEventArgs e)
        {
            if (dgv_groups_data.SelectedIndex != -1)
            {
                Waregroup wg = (Waregroup)dgv_groups_data.SelectedItem;

                if (wg.articleCount > 0)
                {
                    await this.ShowMessageAsync("Ein Fehler ist aufgetreten", "Diese Gruppe kann nicht gelöscht werden, da diese noch Artikel beinhaltet");
                }
                else
                {
                    MetroDialogSettings sets = new()
                    {
                        AffirmativeButtonText = "Ja",
                        NegativeButtonText = "Nein",
                        DefaultButtonFocus = MessageDialogResult.Negative
                    };

                    MessageDialogResult result = await this.ShowMessageAsync("Warengruppe löschen", "Gruppe " + wg.GetName() + " löschen", MessageDialogStyle.AffirmativeAndNegative, sets);
                    if (result == MessageDialogResult.Affirmative)
                    {
                        api.Delete_Waregroup(wg.Get_ID());
                        Get_Waregroups();
                        dgv_groups_data.ItemsSource = Cache.Waregroups;
                    }
                }
            }
            else
            {
                await this.ShowMessageAsync("Ein Fehler ist aufgetreten", "Es wurde keine Warengruppe ausgewählt");
            }
        }
        #endregion
    }
}